#!/usr/bin/perl

use strict;
use warnings;
use lib "${ENV{'HOME'}}/services/perl/lib/perl5/site_perl";
use DBI;

# 20080926 ewn	script that puts stat information into a database. at the moment this works from a sqlite database so not to interfer with mysql. this could be said to be interferable by disk activity. but sqlite was chosen as it's probably less impacting.

my %config;
my %keys;

my $sql_create = "
CREATE TABLE host ( host_id integer not null primary key autoincrement, host_name varchar(9000) );
CREATE TABLE stats ( stats_id integer not null primary key autoincrement, stats_name varchar(9000) not null unique );
CREATE TABLE stats_history ( stats_history_id integer not null primary key autoincrement, stats_id integer not null, stats_history_time timestamp not null default CURRENT_TIMESTAMP, host_id not null, stats_history_value integer not null );
CREATE INDEX host_name_idx on host (host_name);
CREATE INDEX stats_history_host_id_idx on stats_history( host_id );
CREATE INDEX stats_history_stats_id_idx on stats_history( stats_id );
CREATE INDEX stats_history_time_idx on stats_history( stats_history_time );
CREATE INDEX stats_id_name on stats (stats_name);
";

sub set_config {
	if( ! -f "monitor.sqlite" ) {
		$config{'create_tables'} = 1;
	}

	$config{'dbh'} = DBI->connect("dbi:SQLite:dbname=monitor.sqlite", "", "", { AutoCommit => 0 } )
		or die "cannot connect to db:$!";

	if( defined( $config{'create_tables'} ) && $config{'create_tables'} == 1 ) {
		print "Creating tables\n";
		foreach( split( /\n/, $sql_create ) ) {
			my $sth = $config{'dbh'}->prepare( $_ );
			$sth->execute();
		}
		$config{'dbh'}->commit;
		delete $config{'create_tables'};
	}

	$config{'hostname'} = `hostname`;
	chomp $config{'hostname'};
	$config{'hostname'} =~ s/^([^.]+).*$/$1/s;

	open( F, $config{'hostname'} )
		or die( "cannot find local config" );
	while( <F> ) {
		chomp;
		my $l = $_;
		if( $l =~ /^(.*?)\s*=\s*(.*)$/i ) {
			$config{$1} = $2;
		}
	}	
	close(F);
}

sub print_config {
	foreach( sort keys %config ) {
		print $_, "\t[", $config{$_}, "]\n";
	}
}

sub get_key_name {
	my $name = shift;

	if( defined( $keys{$name} ) ) {
		return( $keys{$name} );
	}

	my $sql = "select stats_id,stats_name from stats where stats_name like ?";
	my $sth = $config{'dbh'}->prepare( $sql );
	$sth->execute( $name );

	if( my $hr = $sth->fetchrow_hashref() ) {
		my %h = %$hr;

		$keys{$name} = $h{'stats_id'};
		$sth->finish();
		return( $keys{$name} );
	}
	
	$sql = "insert into stats (stats_name) values (?)";
	$sth = $config{'dbh'}->prepare( $sql );
	$sth->execute( $name );

	$keys{$name} = $config{'dbh'}->last_insert_id( undef, undef, qw( stats stats_id ) );
	return( $keys{$name} );
}

sub insert_to_db {
	my $kname = shift;
	my $kvalue = shift;
	my $knum = get_key_name( $kname );

	my $sql = "insert into stats_history ( stats_id, host_id, stats_history_value ) values ( ?, ?, ? )";
	my $sth = $config{'dbh'}->prepare( $sql );
	$sth->execute( $knum, 1, $kvalue );
}

# do disk stats
sub do_checks_disk {
	my @vals = qw( major minor name rio rmerge rsect ruse wio wmerge wsect wuse running use aveq );
	my $file = "/proc/diskstats";
	open( F, "<$file" ) or die( "cannot open $file:$!" );
	my @lines = <F>;
	close(F);
	chomp @lines;

	foreach my $disk ( split( /:/, $config{'disks'} ) ) {
		my %disk_stats;
		my @l = grep( /\s$disk\s/, @lines );
		foreach( @l ) {
			$_ =~ s/^\s+//;
			my $detail = $_;
			my @v = split( m/\s+/, $detail );

			for( my $i = 0 ; $i<scalar( @vals ) ; $i++ ) {
				$disk_stats{$vals[$i]} = $v[$i];
			}
		}

		foreach( @vals ) {
			print $disk, "\t", $_, "\t", $disk_stats{$_}, "\n" if defined( $disk_stats{$_} );
			insert_to_db( "disk_${disk}_${_}", $disk_stats{$_} );
		}
	}
}

sub do_checks_network {
	 my @vals = qw( bytes_rx packets_rx errs_rx drop_rx fifo_rx frame_rx compressed_rx multicast_rx bytes_tx packets_tx errs_tx drop_tx fifo_tx colls_tx carrier_tx compressed_tx );

	my $file = "/proc/net/dev";
	open( F, "<$file" ) or die( "cannot open $file:$!" );
	my @lines = <F>;
	close(F);
	chomp @lines;

	foreach my $interface ( split( /:/, $config{'interfaces'} ) ) {
		my %interface_stats;
		my @l = grep( /\s+$interface:/, @lines );
		foreach( @l ) {
			my $detail = $_;
			$detail =~ s/^\s+$interface:(.*)/$1/;

			my @v = split( m/\s+/, $detail );

			for( my $i = 0 ; $i<scalar( @vals ) ; $i++ ) {
				$interface_stats{$vals[$i]} = $v[$i];
			}
		}
		
		foreach( @vals ) {
			print $interface, "\t", $_, "\t", $interface_stats{$_}, "\n" if defined( $interface_stats{$_} );
			insert_to_db( "interface_${interface}_${_}", $interface_stats{$_} );
		}
	}
}

sub do_checks_cpu {
	my @vals = qw( cpu user nice system idle iowait irc softirq steal );

	my $file = "/proc/stat";
	open( F, "<$file" ) or die( "cannot open $file:$!" );
	my @lines = <F>;
	close(F);
	chomp @lines;

	foreach my $cpu ( split( /:/, $config{'cpu'} ) ) {
		my %cpu_stats;
		my @l = grep( /\s*$cpu\s+/, @lines );
		foreach( @l ) {
			my $detail = $_;
			#print( $detail, "\n" );

			my @v = split( m/\s+/, $detail );

			for( my $i = 0 ; $i<scalar( @vals ) ; $i++ ) {
				$cpu_stats{$vals[$i]} = $v[$i];
			}
		}
		
		foreach( @vals ) {
			next if( !defined( $cpu_stats{$_} ) );
			print $cpu, "\t", $_, "\t", $cpu_stats{$_}, "\n" if defined( $cpu_stats{$_} );
			insert_to_db( "cpu_${cpu}_${_}", $cpu_stats{$_} );
		}
	}

}

sub do_checks {
	do_checks_disk();	
	do_checks_network();
	do_checks_cpu();
}

sub main {
	set_config();
	print_config();
	do_checks();
	$config{'dbh'}->commit;
}

main;



